@@ -22,20 +22,25 @@
 
 #include "config.h"
 
-#include <dazzle.h>
 #include <glib/gi18n.h>
+
 #include <gtksourceview/gtksource.h>
-#include <libide-plugins.h>
 #include <string.h>
 
+#include <libide-plugins.h>
+
 #include "ide-buffer.h"
 #include "ide-buffer-private.h"
 #include "ide-highlight-engine.h"
 #include "ide-highlight-index.h"
 #include "ide-highlighter.h"
 
-#define HIGHLIGHT_QUANTA_USEC 5000
-#define PRIVATE_TAG_PREFIX    "gb-private-tag"
+#include "cjhtextregionprivate.h"
+
+#define PRIVATE_TAG_PREFIX "Builder"
+
+#define RUN_UNCHECKED GSIZE_TO_POINTER(0)
+#define RUN_CHECKED   GSIZE_TO_POINTER(1)
 
 struct _IdeHighlightEngine
 {
@@ -43,21 +48,22 @@ struct _IdeHighlightEngine
 
   GWeakRef             buffer_wref;
 
-  DzlSignalGroup      *signal_group;
+  IdeSignalGroup      *signal_group;
   IdeHighlighter      *highlighter;
   GSettings           *settings;
 
   IdeExtensionAdapter *extension;
 
-  GtkTextMark         *invalid_begin;
-  GtkTextMark         *invalid_end;
+  CjhTextRegion       *region;
 
   GSList              *private_tags;
   GSList              *public_tags;
 
   gint64               quanta_expiration;
 
-  guint                work_timeout;
+  gsize                work_scheduled;
+
+  guint                commit_funcs_handler;
 
   guint                enabled : 1;
 };
@@ -73,77 +79,17 @@ enum {
 
 static GParamSpec *properties [LAST_PROP];
 
-static gboolean
-get_invalidation_area (GtkTextIter *begin,
-                       GtkTextIter *end)
-{
-  GtkTextIter begin_tmp;
-  GtkTextIter end_tmp;
-
-  g_assert (begin != NULL);
-  g_assert (end != NULL);
-
-  /*
-   * Move to the beginning of line.We dont use gtk_text_iter_backward_line
-   * because if begin is at the beginning of the line we dont want to
-   * move to the previous line
-   */
-  gtk_text_iter_set_line_offset (begin, 0);
-
-  /* Move to the beginning of the next line. */
-  gtk_text_iter_forward_line (end);
-
-  /* Save the original locations. We will need them down the line. */
-  begin_tmp = *begin;
-  end_tmp = *end;
-
-  /*
-   * Fordward begin iter character by character until:
-   * - We reach a non space character
-   * - We reach end iter
-   */
-  while (g_unichar_isspace (gtk_text_iter_get_char (begin)) &&
-         gtk_text_iter_compare (begin, &end_tmp) < 0)
-    gtk_text_iter_forward_char (begin);
-
-
-  /*
-   * If after moving forward the begin iter, we reached the end iter,
-   * there is no need to play with the end iter.
-   */
-  if (gtk_text_iter_compare (begin, end) < 0)
-    {
-      /*
-       * Backward end iter character by character until:
-       * - We reach a non space character
-       * - We reach begin iter
-       */
-      while (g_unichar_isspace (gtk_text_iter_get_char (end)) &&
-             gtk_text_iter_compare (end, &begin_tmp) > 0)
-        gtk_text_iter_backward_char (end);
-
-      /*
-       * If we found the character we are looking for then move one
-       * character forward in order to include it as the last
-       * character of the begin - end range.
-       */
-      if (gtk_text_iter_compare (end, &end_tmp) < 0)
-        gtk_text_iter_forward_char (end);
-    }
-
-  return gtk_text_iter_compare (begin, end) < 0;
-}
-
 static void
 sync_tag_style (GtkSourceStyleScheme *style_scheme,
+                GtkSourceLanguage    *language,
                 GtkTextTag           *tag)
 {
   g_autofree gchar *foreground = NULL;
   g_autofree gchar *background = NULL;
   g_autofree gchar *tag_name = NULL;
-  gchar *style_name = NULL;
-  const gchar *colon;
-  GtkSourceStyle *style;
+  const char *style_name = NULL;
+  GtkSourceStyle *style = NULL;
+  const char *fallback;
   gboolean foreground_set = FALSE;
   gboolean background_set = FALSE;
   gboolean bold = FALSE;
@@ -173,25 +119,29 @@ sync_tag_style (GtkSourceStyleScheme *style_scheme,
   style_name = tag_name;
 
   /*
-   * Check if this is a private tag.A tag is private if it starts with
-   * PRIVATE_TAG_PREFIX "gb-private-tag".
-   * ex: gb-private-tag:c:boolean
+   * Check if this is a private tag. A tag is private if it starts with
+   * PRIVATE_TAG_PREFIX "Builder" such as "Builder:c:boolean".
    * If the tag is private extract the original style name by moving the string
    * strlen (PRIVATE_TAG_PREFIX) + 1 (the colon) characters.
    */
   if (tag_name_len > prefix_len && memcmp (tag_name, PRIVATE_TAG_PREFIX, prefix_len) == 0)
     style_name = tag_name + prefix_len + 1;
 
-  style = gtk_source_style_scheme_get_style (style_scheme, style_name);
-  if (style == NULL && (colon = strchr (style_name, ':')))
+  fallback = style_name;
+  while (style == NULL && fallback != NULL)
     {
-      gchar defname[64];
-      g_snprintf (defname, sizeof defname, "def%s", colon);
-      style = gtk_source_style_scheme_get_style (style_scheme, defname);
-      if (style == NULL)
-        return;
+      if (!(style = gtk_source_style_scheme_get_style (style_scheme, fallback)))
+        {
+          if (language)
+            fallback = gtk_source_language_get_style_fallback (language, fallback);
+          else
+            fallback = NULL;
+        }
     }
 
+  if (style == NULL)
+    return;
+
   g_object_get (style,
                 "background", &background,
                 "background-set", &background_set,
@@ -227,6 +177,7 @@ create_tag_from_style (IdeHighlightEngine *self,
 {
   g_autoptr(IdeBuffer) buffer = NULL;
   GtkSourceStyleScheme *style_scheme;
+  GtkSourceLanguage *language;
   GtkTextTag *tag = NULL;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
@@ -238,8 +189,11 @@ create_tag_from_style (IdeHighlightEngine *self,
     {
       tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer), style_name, NULL);
       gtk_text_tag_set_priority (tag, 0);
+
       style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (buffer));
-      sync_tag_style (style_scheme, tag);
+      language = gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (buffer));
+
+      sync_tag_style (style_scheme, language, tag);
     }
 
   return tag;
@@ -263,7 +217,7 @@ get_tag_from_style (IdeHighlightEngine *self,
     return NULL;
 
   /*
-   * If is private tag prepend the PRIVATE_TAG_PREFIX (gb-private-tag)
+   * If is private tag prepend the PRIVATE_TAG_PREFIX (Builder)
    * to the string.This is used because tag name is the key used
    * for saving tags in GtkTextTagTable and we dont want conflicts between
    * public and private tags.
@@ -310,86 +264,140 @@ ide_highlight_engine_apply_style (const GtkTextIter *begin,
   return IDE_HIGHLIGHT_CONTINUE;
 }
 
+typedef struct
+{
+  gsize offset;
+  gsize length;
+} GetUncheckedRange;
+
+static inline gboolean
+get_unchecked_start_cb (gsize                   offset,
+                        const CjhTextRegionRun *run,
+                        gpointer                user_data)
+{
+  GetUncheckedRange *range = user_data;
+
+  if (range->offset == G_MAXSIZE)
+    {
+      if (run->data == RUN_UNCHECKED)
+        {
+          range->offset = offset;
+          range->length = run->length;
+        }
+
+      return FALSE;
+    }
+  else if (run->data == RUN_UNCHECKED &&
+           range->offset + range->length == offset)
+    {
+      range->length += run->length;
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static gboolean
-ide_highlight_engine_tick (IdeHighlightEngine *self)
+get_next_range (CjhTextRegion *region,
+                GtkTextBuffer *buffer,
+                GtkTextIter   *begin,
+                GtkTextIter   *end)
+{
+  GetUncheckedRange range = {G_MAXSIZE, G_MAXSIZE};
+
+  _cjh_text_region_foreach (region, get_unchecked_start_cb, &range);
+
+  if (range.offset == G_MAXSIZE)
+    return FALSE;
+
+  gtk_text_buffer_get_iter_at_offset (buffer, begin, range.offset);
+  gtk_text_buffer_get_iter_at_offset (buffer, end, range.offset + range.length);
+
+  return TRUE;
+}
+
+static gboolean
+ide_highlight_engine_tick (IdeHighlightEngine *self,
+                           gint64              deadline)
 {
   g_autoptr(GtkTextBuffer) buffer = NULL;
   GtkTextIter iter;
   GtkTextIter invalid_begin;
   GtkTextIter invalid_end;
-  GSList *tags_iter;
 
   IDE_PROBE;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
   g_assert (self->highlighter != NULL);
-  g_assert (self->invalid_begin != NULL);
-  g_assert (self->invalid_end != NULL);
 
-  buffer = g_weak_ref_get (&self->buffer_wref);
-  if (buffer == NULL)
+  if (!(buffer = g_weak_ref_get (&self->buffer_wref)))
     return G_SOURCE_REMOVE;
 
-  self->quanta_expiration = g_get_monotonic_time () + HIGHLIGHT_QUANTA_USEC;
+  self->quanta_expiration = deadline;
 
-  gtk_text_buffer_get_iter_at_mark (buffer, &invalid_begin, self->invalid_begin);
-  gtk_text_buffer_get_iter_at_mark (buffer, &invalid_end, self->invalid_end);
+  if (!get_next_range (self->region, buffer, &invalid_begin, &invalid_end))
+    return G_SOURCE_REMOVE;
+
+again:
+  g_assert (gtk_text_iter_compare (&invalid_begin, &invalid_end) <= 0);
 
   IDE_TRACE_MSG ("Highlight Range [%u:%u,%u:%u] (%s)",
-                 gtk_text_iter_get_line (&invalid_begin),
-                 gtk_text_iter_get_line_offset (&invalid_begin),
-                 gtk_text_iter_get_line (&invalid_end),
-                 gtk_text_iter_get_line_offset (&invalid_end),
+                 gtk_text_iter_get_line (&invalid_begin) + 1,
+                 gtk_text_iter_get_line_offset (&invalid_begin) + 1,
+                 gtk_text_iter_get_line (&invalid_end) + 1,
+                 gtk_text_iter_get_line_offset (&invalid_end) + 1,
                  G_OBJECT_TYPE_NAME (self->highlighter));
 
-  if (gtk_text_iter_compare (&invalid_begin, &invalid_end) >= 0)
-    IDE_GOTO (up_to_date);
-
-  /* Clear all our tags */
-  for (tags_iter = self->private_tags; tags_iter; tags_iter = tags_iter->next)
-    gtk_text_buffer_remove_tag (buffer,
-                                GTK_TEXT_TAG (tags_iter->data),
-                                &invalid_begin,
-                                &invalid_end);
-
   iter = invalid_begin;
 
-  ide_highlighter_update (self->highlighter, ide_highlight_engine_apply_style,
-                          &invalid_begin, &invalid_end, &iter);
+  while (gtk_text_iter_ends_line (&iter))
+    {
+      if (!gtk_text_iter_forward_char (&iter))
+        break;
+    }
+
+  if (gtk_text_iter_compare (&iter, &invalid_end) < 0)
+    ide_highlighter_update (self->highlighter,
+                            self->private_tags,
+                            ide_highlight_engine_apply_style,
+                            &invalid_begin,
+                            &invalid_end,
+                            &iter);
+
+  if (!gtk_text_iter_equal (&iter, &invalid_begin))
+    _cjh_text_region_replace (self->region,
+                              gtk_text_iter_get_offset (&invalid_begin),
+                              gtk_text_iter_get_offset (&iter) - gtk_text_iter_get_offset (&invalid_begin),
+                              RUN_CHECKED);
 
   if (gtk_text_iter_compare (&iter, &invalid_end) >= 0)
-    IDE_GOTO (up_to_date);
+    {
+      if (get_next_range (self->region, buffer, &invalid_begin, &invalid_end))
+        IDE_GOTO (again);
+    }
 
   /* Stop processing until further instruction if no movement was made */
   if (gtk_text_iter_equal (&iter, &invalid_begin))
     return G_SOURCE_REMOVE;
 
-  gtk_text_buffer_move_mark (buffer, self->invalid_begin, &iter);
-
   return G_SOURCE_CONTINUE;
-
-up_to_date:
-  gtk_text_buffer_get_start_iter (buffer, &iter);
-  gtk_text_buffer_move_mark (buffer, self->invalid_begin, &iter);
-  gtk_text_buffer_move_mark (buffer, self->invalid_end, &iter);
-
-  return G_SOURCE_REMOVE;
 }
 
 static gboolean
-ide_highlight_engine_work_timeout_handler (gpointer data)
+ide_highlight_engine_worker (gint64   deadline,
+                             gpointer user_data)
 {
-  IdeHighlightEngine *self = data;
+  IdeHighlightEngine *self = user_data;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
 
   if (self->enabled)
     {
-      if (ide_highlight_engine_tick (self))
+      if (ide_highlight_engine_tick (self, deadline))
         return G_SOURCE_CONTINUE;
     }
 
-  self->work_timeout = 0;
+  self->work_scheduled = 0;
 
   return G_SOURCE_REMOVE;
 }
@@ -401,23 +409,13 @@ ide_highlight_engine_queue_work (IdeHighlightEngine *self)
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
 
-  buffer = g_weak_ref_get (&self->buffer_wref);
-  if (self->highlighter == NULL || buffer == NULL || self->work_timeout != 0)
+  if (self->work_scheduled != 0 || self->highlighter == NULL)
     return;
 
-  /*
-   * NOTE: It would be really nice if we could use the GdkFrameClock here to
-   *       drive the next update instead of a timeout. It's possible that our
-   *       callback could get scheduled right before the frame processing would
-   *       begin. However, since that gets driven by something like a Wayland
-   *       callback, it won't yet be scheduled. So instead our function gets
-   *       called and we potentially cause a frame to drop.
-   */
+  if (!(buffer = g_weak_ref_get (&self->buffer_wref)))
+    return;
 
-  self->work_timeout = gdk_threads_add_idle_full (G_PRIORITY_LOW + 1,
-                                                  ide_highlight_engine_work_timeout_handler,
-                                                  self,
-                                                  NULL);
+  self->work_scheduled = gtk_source_scheduler_add (ide_highlight_engine_worker, self);
 }
 
 /**
@@ -429,8 +427,6 @@ ide_highlight_engine_queue_work (IdeHighlightEngine *self)
  *
  * If they return from their update function without advancing, nothing will
  * happen until they call this method to proceed.
- *
- * Since: 3.32
  */
 void
 ide_highlight_engine_advance (IdeHighlightEngine *self)
@@ -440,51 +436,6 @@ ide_highlight_engine_advance (IdeHighlightEngine *self)
   ide_highlight_engine_queue_work (self);
 }
 
-static gboolean
-invalidate_and_highlight (IdeHighlightEngine *self,
-                          GtkTextIter        *begin,
-                          GtkTextIter        *end)
-{
-  GtkTextBuffer *text_buffer;
-
-  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
-  g_assert (begin != NULL);
-  g_assert (end != NULL);
-
-  if (!self->enabled)
-    return FALSE;
-
-  text_buffer = gtk_text_iter_get_buffer (begin);
-
-  if (get_invalidation_area (begin, end))
-    {
-      GtkTextIter begin_tmp;
-      GtkTextIter end_tmp;
-
-      gtk_text_buffer_get_iter_at_mark (text_buffer, &begin_tmp, self->invalid_begin);
-      gtk_text_buffer_get_iter_at_mark (text_buffer, &end_tmp, self->invalid_end);
-
-      if (gtk_text_iter_equal (&begin_tmp, &end_tmp))
-        {
-          gtk_text_buffer_move_mark (text_buffer, self->invalid_begin, begin);
-          gtk_text_buffer_move_mark (text_buffer, self->invalid_end, end);
-        }
-      else
-        {
-          if (gtk_text_iter_compare (begin, &begin_tmp) < 0)
-            gtk_text_buffer_move_mark (text_buffer, self->invalid_begin, begin);
-          if (gtk_text_iter_compare (end, &end_tmp) > 0)
-            gtk_text_buffer_move_mark (text_buffer, self->invalid_end, end);
-        }
-
-      ide_highlight_engine_queue_work (self);
-
-      return TRUE;
-    }
-
-  return FALSE;
-}
-
 static void
 ide_highlight_engine_reload (IdeHighlightEngine *self)
 {
@@ -497,23 +448,15 @@ ide_highlight_engine_reload (IdeHighlightEngine *self)
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
 
-  dzl_clear_source (&self->work_timeout);
+  gtk_source_scheduler_clear (&self->work_scheduled);
 
-  buffer = g_weak_ref_get (&self->buffer_wref);
-  if (buffer == NULL)
+  if (!(buffer = g_weak_ref_get (&self->buffer_wref)))
     IDE_EXIT;
 
   gtk_text_buffer_get_bounds (buffer, &begin, &end);
+  ide_highlight_engine_invalidate (self, &begin, &end);
 
-  /*
-   * Invalidate the whole buffer.
-   */
-  gtk_text_buffer_move_mark (buffer, self->invalid_begin, &begin);
-  gtk_text_buffer_move_mark (buffer, self->invalid_end, &end);
-
-  /*
-   * Remove our highlight tags from the buffer.
-   */
+  /* Remove our highlight tags from the buffer. */
   for (iter = self->private_tags; iter; iter = iter->next)
     gtk_text_buffer_remove_tag (buffer, iter->data, &begin, &end);
   g_clear_pointer (&self->private_tags, g_slist_free);
@@ -550,71 +493,6 @@ ide_highlight_engine_set_highlighter (IdeHighlightEngine *self,
     }
 }
 
-static void
-ide_highlight_engine__buffer_insert_text_cb (IdeHighlightEngine *self,
-                                             GtkTextIter        *location,
-                                             gchar              *text,
-                                             gint                len,
-                                             IdeBuffer          *buffer)
-{
-  GtkTextIter begin;
-  GtkTextIter end;
-
-  IDE_ENTRY;
-
-  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
-  g_assert (location);
-  g_assert (text);
-  g_assert (IDE_IS_BUFFER (buffer));
-
-  if (!self->enabled)
-    IDE_EXIT;
-
-  /*
-   * Backward the begin iter len characters from location
-   * (location points to the end of the string) in order to get
-   * the iter position where our inserted text was started.
-   */
-  begin = *location;
-  gtk_text_iter_backward_chars (&begin, g_utf8_strlen (text, len));
-
-  end = *location;
-
-  invalidate_and_highlight (self, &begin, &end);
-
-  IDE_EXIT;
-}
-
-static void
-ide_highlight_engine__buffer_delete_range_cb (IdeHighlightEngine *self,
-                                              GtkTextIter        *range_begin,
-                                              GtkTextIter        *range_end,
-                                              IdeBuffer          *buffer)
-{
-  GtkTextIter begin;
-  GtkTextIter end;
-
-  IDE_ENTRY;
-
-  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
-  g_assert (range_begin);
-  g_assert (IDE_IS_BUFFER (buffer));
-
-  if (!self->enabled)
-    IDE_EXIT;
-
-  /*
-   * No need to use the range_end since everything that
-   * was after range_end will now be after range_begin
-   */
-  begin = *range_begin;
-  end = *range_begin;
-
-  invalidate_and_highlight (self, &begin, &end);
-
-  IDE_EXIT;
-}
-
 static void
 ide_highlight_engine__notify_language_cb (IdeHighlightEngine *self,
                                           GParamSpec         *pspec,
@@ -641,17 +519,19 @@ ide_highlight_engine__notify_style_scheme_cb (IdeHighlightEngine *self,
                                               IdeBuffer          *buffer)
 {
   GtkSourceStyleScheme *style_scheme;
+  GtkSourceLanguage *language;
   GSList *iter;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
   g_assert (IDE_IS_BUFFER (buffer));
 
   style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (buffer));
+  language = gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (buffer));
 
   for (iter = self->private_tags; iter; iter = iter->next)
-    sync_tag_style (style_scheme, iter->data);
+    sync_tag_style (style_scheme, language, iter->data);
   for (iter = self->public_tags; iter; iter = iter->next)
-    sync_tag_style (style_scheme, iter->data);
+    sync_tag_style (style_scheme, language, iter->data);
 }
 
 void
@@ -677,36 +557,126 @@ ide_highlight_engine_clear (IdeHighlightEngine *self)
     }
 }
 
+static void
+mark_unchecked (IdeHighlightEngine *self,
+                IdeBuffer          *buffer,
+                guint               offset,
+                guint               length)
+{
+  GtkTextIter begin, end;
+
+  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
+
+  gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (buffer), &begin, offset);
+  gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (buffer), &end, offset + length);
+
+  while (!gtk_text_iter_starts_line (&begin))
+    {
+      gunichar ch;
+
+      if (!gtk_text_iter_backward_char (&begin))
+        break;
+
+      ch = gtk_text_iter_get_char (&begin);
+
+      if (ch != '_' && !g_unichar_isalnum (ch))
+        break;
+    }
+
+  while (!gtk_text_iter_ends_line (&end))
+    {
+      gunichar ch;
+
+      if (!gtk_text_iter_forward_char (&end))
+        break;
+
+      ch = gtk_text_iter_get_char (&end);
+
+      if (ch != '_' && !g_unichar_isalnum (ch))
+        break;
+    }
+
+  ide_highlight_engine_invalidate (self, &begin, &end);
+}
+
+static void
+ide_highlight_engine_insert_before_cb (IdeBuffer *buffer,
+                                       guint      offset,
+                                       guint      length,
+                                       gpointer   user_data)
+{
+  IdeHighlightEngine *self = user_data;
+
+  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
+
+  if (self->enabled)
+    _cjh_text_region_insert (self->region, offset, length, RUN_UNCHECKED);
+}
+
+static void
+ide_highlight_engine_insert_after_cb (IdeBuffer *buffer,
+                                      guint      offset,
+                                      guint      length,
+                                      gpointer   user_data)
+{
+  IdeHighlightEngine *self = user_data;
+
+  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
+
+  if (self->enabled)
+    mark_unchecked (self, buffer, offset, length);
+}
+
+static void
+ide_highlight_engine_delete_before_cb (IdeBuffer *buffer,
+                                       guint      offset,
+                                       guint      length,
+                                       gpointer   user_data)
+{
+  IdeHighlightEngine *self = user_data;
+
+  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
+
+  if (self->enabled)
+    _cjh_text_region_remove (self->region, offset, length);
+}
+
+static void
+ide_highlight_engine_delete_after_cb (IdeBuffer *buffer,
+                                      guint      offset,
+                                      guint      length,
+                                      gpointer   user_data)
+{
+  IdeHighlightEngine *self = user_data;
+
+  g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
+
+  if (self->enabled)
+    mark_unchecked (self, buffer, offset, 0);
+}
+
 static void
 ide_highlight_engine__bind_buffer_cb (IdeHighlightEngine *self,
                                       IdeBuffer          *buffer,
-                                      DzlSignalGroup     *group)
+                                      IdeSignalGroup     *group)
 {
   GtkTextBuffer *text_buffer = (GtkTextBuffer *)buffer;
-  GtkTextIter begin;
-  GtkTextIter end;
 
   IDE_ENTRY;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
   g_assert (IDE_IS_BUFFER (buffer));
-  g_assert (DZL_IS_SIGNAL_GROUP (group));
-  g_assert (self->invalid_begin == NULL);
-  g_assert (self->invalid_end == NULL);
+  g_assert (IDE_IS_SIGNAL_GROUP (group));
 
   g_weak_ref_set (&self->buffer_wref, buffer);
 
-  gtk_text_buffer_get_bounds (text_buffer, &begin, &end);
-
-  self->invalid_begin = gtk_text_buffer_create_mark (text_buffer, NULL, &begin, TRUE);
-  self->invalid_end = gtk_text_buffer_create_mark (text_buffer, NULL, &end, FALSE);
-
-  /* We can hold a full reference to the text marks, without
-   * taking a reference to the buffer. We want to avoid a reference
-   * to the buffer for cyclic reasons.
-   */
-  g_object_ref (self->invalid_begin);
-  g_object_ref (self->invalid_end);
+  self->commit_funcs_handler =
+    ide_buffer_add_commit_funcs (IDE_BUFFER (text_buffer),
+                                 ide_highlight_engine_insert_before_cb,
+                                 ide_highlight_engine_insert_after_cb,
+                                 ide_highlight_engine_delete_before_cb,
+                                 ide_highlight_engine_delete_after_cb,
+                                 self, NULL);
 
   ide_highlight_engine__notify_style_scheme_cb (self, NULL, buffer);
   ide_highlight_engine__notify_language_cb (self, NULL, buffer);
@@ -718,18 +688,22 @@ ide_highlight_engine__bind_buffer_cb (IdeHighlightEngine *self,
 
 static void
 ide_highlight_engine__unbind_buffer_cb (IdeHighlightEngine  *self,
-                                        DzlSignalGroup      *group)
+                                        IdeSignalGroup      *group)
 {
   g_autoptr(GtkTextBuffer) text_buffer = NULL;
+  gsize length;
 
   IDE_ENTRY;
 
   g_assert (IDE_IS_HIGHLIGHT_ENGINE (self));
-  g_assert (DZL_IS_SIGNAL_GROUP (group));
+  g_assert (IDE_IS_SIGNAL_GROUP (group));
 
   text_buffer = g_weak_ref_get (&self->buffer_wref);
 
-  dzl_clear_source (&self->work_timeout);
+  gtk_source_scheduler_clear (&self->work_scheduled);
+
+  if ((length = _cjh_text_region_get_length (self->region)))
+    _cjh_text_region_remove (self->region, 0, length - 1);
 
   if (text_buffer != NULL)
     {
@@ -739,10 +713,14 @@ ide_highlight_engine__unbind_buffer_cb (IdeHighlightEngine  *self,
       GtkTextIter begin;
       GtkTextIter end;
 
-      tag_table = gtk_text_buffer_get_tag_table (text_buffer);
+      if (self->commit_funcs_handler)
+        {
+          ide_buffer_remove_commit_funcs (IDE_BUFFER (text_buffer),
+                                          self->commit_funcs_handler);
+          self->commit_funcs_handler = 0;
+        }
 
-      gtk_text_buffer_delete_mark (text_buffer, self->invalid_begin);
-      gtk_text_buffer_delete_mark (text_buffer, self->invalid_end);
+      tag_table = gtk_text_buffer_get_tag_table (text_buffer);
 
       gtk_text_buffer_get_bounds (text_buffer, &begin, &end);
 
@@ -765,9 +743,6 @@ ide_highlight_engine__unbind_buffer_cb (IdeHighlightEngine  *self,
   g_clear_pointer (&self->public_tags, g_slist_free);
   g_clear_pointer (&self->private_tags, g_slist_free);
 
-  g_clear_object (&self->invalid_begin);
-  g_clear_object (&self->invalid_end);
-
   IDE_EXIT;
 }
 
@@ -781,7 +756,7 @@ ide_highlight_engine_set_buffer (IdeHighlightEngine *self,
   /* We can get GtkSourceBuffer intermittently here. */
   if (!buffer || IDE_IS_BUFFER (buffer))
     {
-      dzl_signal_group_set_target (self->signal_group, buffer);
+      ide_signal_group_set_target (self->signal_group, buffer);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_BUFFER]);
     }
 }
@@ -859,6 +834,7 @@ ide_highlight_engine_dispose (GObject *object)
   g_clear_object (&self->extension);
   g_clear_object (&self->highlighter);
   g_clear_object (&self->settings);
+  g_clear_pointer (&self->region, _cjh_text_region_free);
 
   G_OBJECT_CLASS (ide_highlight_engine_parent_class)->dispose (object);
 }
@@ -952,27 +928,17 @@ ide_highlight_engine_init (IdeHighlightEngine *self)
 
   self->settings = g_settings_new ("org.gnome.builder.code-insight");
   self->enabled = g_settings_get_boolean (self->settings, "semantic-highlighting");
-  self->signal_group = dzl_signal_group_new (IDE_TYPE_BUFFER);
+  self->signal_group = ide_signal_group_new (IDE_TYPE_BUFFER);
 
-  dzl_signal_group_connect_object (self->signal_group,
-                                   "insert-text",
-                                   G_CALLBACK (ide_highlight_engine__buffer_insert_text_cb),
-                                   self,
-                                   G_CONNECT_SWAPPED | G_CONNECT_AFTER);
+  self->region = _cjh_text_region_new (NULL, NULL);
 
-  dzl_signal_group_connect_object (self->signal_group,
-                                   "delete-range",
-                                   G_CALLBACK (ide_highlight_engine__buffer_delete_range_cb),
-                                   self,
-                                   G_CONNECT_SWAPPED | G_CONNECT_AFTER);
-
-  dzl_signal_group_connect_object (self->signal_group,
+  ide_signal_group_connect_object (self->signal_group,
                                    "notify::language",
                                    G_CALLBACK (ide_highlight_engine__notify_language_cb),
                                    self,
                                    G_CONNECT_SWAPPED);
 
-  dzl_signal_group_connect_object (self->signal_group,
+  ide_signal_group_connect_object (self->signal_group,
                                    "notify::style-scheme",
                                    G_CALLBACK (ide_highlight_engine__notify_style_scheme_cb),
                                    self,
@@ -1022,8 +988,6 @@ ide_highlight_engine_new (IdeBuffer *buffer)
  * Gets the IdeHighlightEngine:highlighter property.
  *
  * Returns: (transfer none): An #IdeHighlighter.
- *
- * Since: 3.32
  */
 IdeHighlighter *
 ide_highlight_engine_get_highlighter (IdeHighlightEngine *self)
@@ -1040,8 +1004,6 @@ ide_highlight_engine_get_highlighter (IdeHighlightEngine *self)
  * Gets the IdeHighlightEngine:buffer property.
  *
  * Returns: (transfer none): An #IdeBuffer.
- *
- * Since: 3.32
  */
 IdeBuffer *
 ide_highlight_engine_get_buffer (IdeHighlightEngine *self)
@@ -1062,25 +1024,37 @@ void
 ide_highlight_engine_rebuild (IdeHighlightEngine *self)
 {
   g_autoptr(GtkTextBuffer) buffer = NULL;
+  GtkTextIter begin, end;
+  gsize length;
 
   IDE_ENTRY;
 
   g_return_if_fail (IDE_IS_HIGHLIGHT_ENGINE (self));
 
-  buffer = g_weak_ref_get (&self->buffer_wref);
+  if (!self->enabled)
+    IDE_EXIT;
 
-  if (buffer != NULL)
+  if ((buffer = g_weak_ref_get (&self->buffer_wref)))
     {
-      GtkTextIter begin;
-      GtkTextIter end;
+      /* We remove using the known length from the region */
+      if ((length = _cjh_text_region_get_length (self->region)) > 0)
+        _cjh_text_region_remove (self->region, 0, length - 1);
 
+      /* We add using the length from the buffer because if we were not
+       * enabled previously, the textregion would be empty.
+       */
       gtk_text_buffer_get_bounds (buffer, &begin, &end);
-      gtk_text_buffer_move_mark (buffer, self->invalid_begin, &begin);
-      gtk_text_buffer_move_mark (buffer, self->invalid_end, &end);
+      if (!gtk_text_iter_equal (&begin, &end))
+        {
+          length = gtk_text_iter_get_offset (&end) - gtk_text_iter_get_offset (&begin);
 
-      ide_highlight_engine_queue_work (self);
+          if (length > 0)
+            _cjh_text_region_insert (self->region, 0, length, RUN_UNCHECKED);
+        }
     }
 
+  ide_highlight_engine_queue_work (self);
+
   IDE_EXIT;
 }
 
@@ -1099,40 +1073,27 @@ ide_highlight_engine_rebuild (IdeHighlightEngine *self)
  * Updating the invalidated region of the buffer may take some time, as it is
  * important that the highlighter does not block for more than 1-2 milliseconds
  * to avoid dropping frames.
- *
- * Since: 3.32
  */
 void
 ide_highlight_engine_invalidate (IdeHighlightEngine *self,
                                  const GtkTextIter  *begin,
                                  const GtkTextIter  *end)
 {
-  GtkTextBuffer *buffer;
-  GtkTextIter mark_begin;
-  GtkTextIter mark_end;
+  guint offset;
+  guint length;
 
   IDE_ENTRY;
 
   g_return_if_fail (IDE_IS_HIGHLIGHT_ENGINE (self));
-  g_return_if_fail (begin != NULL);
-  g_return_if_fail (end != NULL);
+  g_return_if_fail (gtk_text_iter_compare (begin, end) <= 0);
 
-  buffer = gtk_text_iter_get_buffer (begin);
+  offset = gtk_text_iter_get_offset (begin);
+  length = gtk_text_iter_get_offset (end) - gtk_text_iter_get_offset (begin);
 
-  gtk_text_buffer_get_iter_at_mark (buffer, &mark_begin, self->invalid_begin);
-  gtk_text_buffer_get_iter_at_mark (buffer, &mark_end, self->invalid_end);
-
-  if (gtk_text_iter_equal (&mark_begin, &mark_end))
+  if (length > 0)
     {
-      gtk_text_buffer_move_mark (buffer, self->invalid_begin, begin);
-      gtk_text_buffer_move_mark (buffer, self->invalid_end, end);
-    }
-  else
-    {
-      if (gtk_text_iter_compare (begin, &mark_begin) < 0)
-        gtk_text_buffer_move_mark (buffer, self->invalid_begin, begin);
-      if (gtk_text_iter_compare (end, &mark_end) > 0)
-        gtk_text_buffer_move_mark (buffer, self->invalid_end, end);
+      _cjh_text_region_remove (self->region, offset, length);
+      _cjh_text_region_insert (self->region, offset, length, RUN_UNCHECKED);
     }
 
   ide_highlight_engine_queue_work (self);
@@ -1148,8 +1109,6 @@ ide_highlight_engine_invalidate (IdeHighlightEngine *self,
  * A #GtkTextTag for @style_name.
  *
  * Returns: (transfer none): a #GtkTextTag.
- *
- * Since: 3.32
  */
 GtkTextTag *
 ide_highlight_engine_get_style (IdeHighlightEngine *self,
@@ -1163,7 +1122,7 @@ ide_highlight_engine_pause (IdeHighlightEngine *self)
 {
   g_return_if_fail (IDE_IS_HIGHLIGHT_ENGINE (self));
 
-  dzl_signal_group_block (self->signal_group);
+  ide_signal_group_block (self->signal_group);
 }
 
 void
@@ -1174,7 +1133,7 @@ ide_highlight_engine_unpause (IdeHighlightEngine *self)
   g_return_if_fail (IDE_IS_HIGHLIGHT_ENGINE (self));
   g_return_if_fail (self->signal_group != NULL);
 
-  dzl_signal_group_unblock (self->signal_group);
+  ide_signal_group_unblock (self->signal_group);
 
   buffer = g_weak_ref_get (&self->buffer_wref);
 
