@@ -22,21 +22,21 @@
 
 #include "config.h"
 
-#include <dazzle.h>
 #include <glib/gi18n.h>
-#include <libide-code.h>
-#include <libide-plugins.h>
-#include <libide-threading.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <libide-code.h>
+#include <libide-plugins.h>
+#include <libide-threading.h>
+
 #include "ide-buffer-private.h"
 
-#include "ide-debug-manager.h"
+#include "ide-debug-manager-private.h"
 #include "ide-debugger.h"
 #include "ide-debugger-private.h"
 
-#define TAG_CURRENT_BKPT "debugger::current-breakpoint"
+#define TAG_CURRENT_BKPT "-Builder:current-breakpoint"
 
 struct _IdeDebugManager
 {
@@ -44,8 +44,7 @@ struct _IdeDebugManager
 
   GHashTable     *breakpoints;
   IdeDebugger    *debugger;
-  DzlSignalGroup *debugger_signals;
-  IdeRunner      *runner;
+  IdeSignalGroup *debugger_signals;
   GQueue          pending_breakpoints;
   GPtrArray      *supported_languages;
 
@@ -56,8 +55,9 @@ typedef struct
 {
   IdeDebugManager *self;
   IdeDebugger     *debugger;
-  IdeRunner       *runner;
-  gint             priority;
+  IdePipeline     *pipeline;
+  IdeRunCommand   *run_command;
+  int              priority;
 } DebuggerLookup;
 
 enum {
@@ -77,7 +77,7 @@ enum {
 static GParamSpec *properties [N_PROPS];
 static guint signals [N_SIGNALS];
 
-G_DEFINE_TYPE (IdeDebugManager, ide_debug_manager, IDE_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (IdeDebugManager, ide_debug_manager, IDE_TYPE_OBJECT)
 
 static gint
 compare_language_id (gconstpointer a,
@@ -124,8 +124,6 @@ ide_debug_manager_notify_buffer (IdeDebugManager        *self,
  * return %NULL in this case.
  *
  * Returns: %TRUE if the language is supported; otherwise %FALSE.
- *
- * Since: 3.32
  */
 gboolean
 ide_debug_manager_supports_language (IdeDebugManager *self,
@@ -283,9 +281,6 @@ ide_debug_manager_debugger_stopped (IdeDebugManager       *self,
     case IDE_DEBUGGER_STOP_EXITED:
     case IDE_DEBUGGER_STOP_EXITED_NORMALLY:
     case IDE_DEBUGGER_STOP_EXITED_SIGNALED:
-      /* Cleanup any lingering debugger process */
-      if (self->runner != NULL)
-        ide_runner_force_quit (self->runner);
       break;
 
     case IDE_DEBUGGER_STOP_BREAKPOINT_HIT:
@@ -542,13 +537,28 @@ ide_debug_manager_dispose (GObject *object)
   g_queue_clear (&self->pending_breakpoints);
 
   g_hash_table_remove_all (self->breakpoints);
-  dzl_signal_group_set_target (self->debugger_signals, NULL);
+  ide_signal_group_set_target (self->debugger_signals, NULL);
   ide_clear_and_destroy_object (&self->debugger);
-  ide_clear_and_destroy_object (&self->runner);
 
   G_OBJECT_CLASS (ide_debug_manager_parent_class)->dispose (object);
 }
 
+static void
+ide_debug_manager_parent_set (IdeObject *object,
+                              IdeObject *parent)
+{
+  g_autoptr(IdeContext) context = NULL;
+
+  g_assert (IDE_IS_OBJECT (object));
+  g_assert (!parent || IDE_IS_OBJECT (parent));
+
+  if (!parent)
+    return;
+
+  context = ide_object_ref_context (object);
+  ide_context_register_settings (context, "org.gnome.builder.debug");
+}
+
 static void
 ide_debug_manager_finalize (GObject *object)
 {
@@ -556,6 +566,7 @@ ide_debug_manager_finalize (GObject *object)
 
   g_clear_object (&self->debugger_signals);
   g_clear_pointer (&self->breakpoints, g_hash_table_unref);
+  g_clear_pointer (&self->supported_languages, g_ptr_array_unref);
 
   G_OBJECT_CLASS (ide_debug_manager_parent_class)->finalize (object);
 }
@@ -587,11 +598,14 @@ static void
 ide_debug_manager_class_init (IdeDebugManagerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
   object_class->dispose = ide_debug_manager_dispose;
   object_class->finalize = ide_debug_manager_finalize;
   object_class->get_property = ide_debug_manager_get_property;
 
+  i_object_class->parent_set = ide_debug_manager_parent_set;
+
   /**
    * IdeDebugManager:active:
    *
@@ -599,8 +613,6 @@ ide_debug_manager_class_init (IdeDebugManagerClass *klass)
    *
    * This can be used to determine if the controls should be made visible
    * in the workbench.
-   *
-   * Since: 3.32
    */
   properties [PROP_ACTIVE] =
     g_param_spec_boolean ("active",
@@ -625,8 +637,6 @@ ide_debug_manager_class_init (IdeDebugManagerClass *klass)
    *
    * The "breakpoint-added" signal is emitted when a new breakpoint has
    * been registered by the debugger.
-   *
-   * Since: 3.32
    */
   signals [BREAKPOINT_ADDED] =
     g_signal_new_class_handler ("breakpoint-added",
@@ -643,8 +653,6 @@ ide_debug_manager_class_init (IdeDebugManagerClass *klass)
    *
    * The "breakpoint-removed" signal is emitted when a new breakpoint has been
    * removed by the debugger.
-   *
-   * Since: 3.32
    */
   signals [BREAKPOINT_REMOVED] =
     g_signal_new_class_handler ("breakpoint-removed",
@@ -666,8 +674,6 @@ ide_debug_manager_class_init (IdeDebugManagerClass *klass)
    * on the #IdeDebugger itself.
    *
    * See also: #IdeDebugManager:debugger
-   *
-   * Since: 3.32
    */
   signals [BREAKPOINT_REACHED] =
     g_signal_new_class_handler ("breakpoint-reached",
@@ -693,55 +699,34 @@ ide_debug_manager_init (IdeDebugManager *self)
                                              g_object_unref,
                                              g_object_unref);
 
-  self->debugger_signals = dzl_signal_group_new (IDE_TYPE_DEBUGGER);
+  self->debugger_signals = ide_signal_group_new (IDE_TYPE_DEBUGGER);
 
-  dzl_signal_group_connect_swapped (self->debugger_signals,
+  ide_signal_group_connect_swapped (self->debugger_signals,
                                     "stopped",
                                     G_CALLBACK (ide_debug_manager_debugger_stopped),
                                     self);
 
-  dzl_signal_group_connect_swapped (self->debugger_signals,
+  ide_signal_group_connect_swapped (self->debugger_signals,
                                     "running",
                                     G_CALLBACK (ide_debug_manager_debugger_running),
                                     self);
 
-  dzl_signal_group_connect_swapped (self->debugger_signals,
+  ide_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-added",
                                     G_CALLBACK (ide_debug_manager_breakpoint_added),
                                     self);
 
-  dzl_signal_group_connect_swapped (self->debugger_signals,
+  ide_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-modified",
                                     G_CALLBACK (ide_debug_manager_breakpoint_modified),
                                     self);
 
-  dzl_signal_group_connect_swapped (self->debugger_signals,
+  ide_signal_group_connect_swapped (self->debugger_signals,
                                     "breakpoint-removed",
                                     G_CALLBACK (ide_debug_manager_breakpoint_removed),
                                     self);
 }
 
-static gboolean
-debugger_supports_language (PeasPluginInfo *plugin_info,
-                            const gchar    *language)
-{
-  const gchar *supported;
-
-  supported = peas_plugin_info_get_external_data (plugin_info, "Debugger-Languages");
-
-  if (supported != NULL)
-    {
-      g_auto(GStrv) languages = g_strsplit (supported, ",", 0);
-      for (guint i = 0; languages[i]; i++)
-        {
-          if (g_strcmp0 (languages[i], language) == 0)
-            return TRUE;
-        }
-    }
-
-  return FALSE;
-}
-
 static void
 debugger_lookup (PeasExtensionSet *set,
                  PeasPluginInfo   *plugin_info,
@@ -759,18 +744,8 @@ debugger_lookup (PeasExtensionSet *set,
 
   ide_object_append (IDE_OBJECT (lookup->self), IDE_OBJECT (debugger));
 
-  if (ide_debugger_supports_runner (debugger, lookup->runner, &priority))
+  if (ide_debugger_supports_run_command (debugger, lookup->pipeline, lookup->run_command, &priority))
     {
-      IdeBuildTarget *build_target = ide_runner_get_build_target (lookup->runner);
-
-      if (build_target != NULL)
-        {
-          g_autofree gchar *language = ide_build_target_get_language (build_target);
-
-          if (!debugger_supports_language (plugin_info, language))
-            goto failure;
-        }
-
       if (lookup->debugger == NULL || priority < lookup->priority)
         {
           ide_clear_and_destroy_object (&lookup->debugger);
@@ -780,35 +755,25 @@ debugger_lookup (PeasExtensionSet *set,
         }
     }
 
-failure:
   ide_object_destroy (IDE_OBJECT (debugger));
 }
 
-/**
- * ide_debug_manager_find_debugger:
- * @self: a #IdeDebugManager
- * @runner: An #IdeRunner
- *
- * Locates a debugger for the given runner, or %NULL if no debugger
- * supports the runner.
- *
- * Returns: (transfer full) (nullable): An #IdeDebugger or %NULL
- *
- * Since: 3.32
- */
-IdeDebugger *
+static IdeDebugger *
 ide_debug_manager_find_debugger (IdeDebugManager *self,
-                                 IdeRunner       *runner)
+                                 IdePipeline     *pipeline,
+                                 IdeRunCommand   *run_command)
 {
   g_autoptr(PeasExtensionSet) set = NULL;
   DebuggerLookup lookup;
 
-  g_return_val_if_fail (IDE_IS_DEBUG_MANAGER (self), NULL);
-  g_return_val_if_fail (IDE_IS_RUNNER (runner), NULL);
+  g_assert (IDE_IS_DEBUG_MANAGER (self));
+  g_assert (IDE_IS_PIPELINE (pipeline));
+  g_assert (IDE_IS_RUN_COMMAND (run_command));
 
   lookup.self = self;
   lookup.debugger = NULL;
-  lookup.runner = runner;
+  lookup.run_command = run_command;
+  lookup.pipeline = pipeline;
   lookup.priority = G_MAXINT;
 
   set = peas_extension_set_new (peas_engine_get_default (),
@@ -924,34 +889,38 @@ ide_debug_manager_sync_breakpoints (IdeDebugManager *self)
   IDE_EXIT;
 }
 
-static void
-ide_debug_manager_runner_spawned (IdeDebugManager *self,
-                                  const gchar     *identifier,
-                                  IdeRunner       *runner)
+void
+_ide_debug_manager_started (IdeDebugManager *self)
 {
+  IDE_ENTRY;
+
   g_assert (IDE_IS_DEBUG_MANAGER (self));
-  g_assert (identifier != NULL);
-  g_assert (IDE_IS_RUNNER (runner));
+
+  if (self->debugger == NULL)
+    IDE_EXIT;
 
   ide_debug_manager_sync_breakpoints (self);
+
+  IDE_EXIT;
 }
 
-static void
-ide_debug_manager_runner_exited (IdeDebugManager *self,
-                                 IdeRunner       *runner)
+void
+_ide_debug_manager_stopped (IdeDebugManager *self)
 {
   g_autoptr(IdeDebugger) debugger = NULL;
-  g_autoptr(IdeRunner) hold_runner = NULL;
+
+  IDE_ENTRY;
 
   g_assert (IDE_IS_DEBUG_MANAGER (self));
-  g_assert (IDE_IS_RUNNER (runner));
+
+  if (self->debugger == NULL)
+    IDE_EXIT;
 
   /*
    * Keep debugger alive so that listeners to :debugger property can
    * properly disconnect signals when we clear the debugger instance.
    */
   debugger = g_steal_pointer (&self->debugger);
-  hold_runner = g_steal_pointer (&self->runner);
 
   ide_debug_manager_set_active (self, FALSE);
   ide_debug_manager_reset_breakpoints (self);
@@ -960,96 +929,70 @@ ide_debug_manager_runner_exited (IdeDebugManager *self,
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_DEBUGGER]);
 
   ide_clear_and_destroy_object (&debugger);
-  ide_clear_and_destroy_object (&hold_runner);
+
+  IDE_EXIT;
 }
 
-/**
- * ide_debug_manager_start:
+/* _ide_debug_manager_prepare:
  * @self: an #IdeDebugManager
- * @runner: an #IdeRunner
+ * @pipeline: an #IdePipeline
+ * @run_command: an #IdeRunCommand
+ * @run_context: an #IdeRunContext
  * @error: A location for an @error
  *
- * Attempts to start a runner using a discovered debugger backend.
+ * Locates a suitable debugger and prepares it.
+ *
+ * A suitable debugger is located and prepared to run so that when
+ * @run_context is spawned, the debug manager can communicate with
+ * the inferior process.
  *
  * Returns: %TRUE if successful; otherwise %FALSE and @error is set.
- *
- * Since: 3.32
  */
 gboolean
-ide_debug_manager_start (IdeDebugManager  *self,
-                         IdeRunner        *runner,
-                         GError          **error)
+_ide_debug_manager_prepare (IdeDebugManager  *self,
+                            IdePipeline      *pipeline,
+                            IdeRunCommand    *run_command,
+                            IdeRunContext    *run_context,
+                            GError          **error)
 {
   g_autoptr(IdeDebugger) debugger = NULL;
-  IdeEnvironment *environment;
-  gboolean ret = FALSE;
+  g_autoptr(IdeSettings) settings = NULL;
+  IdeContext *context;
 
   IDE_ENTRY;
 
   g_return_val_if_fail (IDE_IS_DEBUG_MANAGER (self), FALSE);
-  g_return_val_if_fail (IDE_IS_RUNNER (runner), FALSE);
+  g_return_val_if_fail (IDE_IS_RUN_COMMAND (run_command), FALSE);
+  g_return_val_if_fail (IDE_IS_RUN_CONTEXT (run_context), FALSE);
   g_return_val_if_fail (self->debugger == NULL, FALSE);
 
-  debugger = ide_debug_manager_find_debugger (self, runner);
+  context = ide_object_get_context (IDE_OBJECT (self));
+  settings = ide_context_ref_settings (context, "org.gnome.builder.debug");
 
-  if (debugger == NULL)
+  if (!(debugger = ide_debug_manager_find_debugger (self, pipeline, run_command)))
     {
-      ide_runner_set_failed (runner, TRUE);
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_NOT_SUPPORTED,
-                   _("A suitable debugger was not found."));
-      IDE_GOTO (failure);
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_NOT_SUPPORTED,
+                           _("A suitable debugger could not be found."));
+      IDE_RETURN (FALSE);
     }
 
-  environment = ide_runner_get_environment (runner);
-  ide_environment_setenv (environment, "G_MESSAGES_DEBUG", "all");
-  ide_debugger_prepare (debugger, runner);
+  ide_debugger_prepare_for_run (debugger, pipeline, run_context);
 
-  g_signal_connect_object (runner,
-                           "spawned",
-                           G_CALLBACK (ide_debug_manager_runner_spawned),
-                           self,
-                           G_CONNECT_SWAPPED);
+  if (ide_settings_get_boolean (settings, "insert-breakpoint-at-criticals"))
+    ide_run_context_setenv (run_context, "G_DEBUG", "fatal-criticals");
+  else if (ide_settings_get_boolean (settings, "insert-breakpoint-at-warnings"))
+    ide_run_context_setenv (run_context, "G_DEBUG", "fatal-warnings");
 
-  g_signal_connect_object (runner,
-                           "exited",
-                           G_CALLBACK (ide_debug_manager_runner_exited),
-                           self,
-                           G_CONNECT_SWAPPED | G_CONNECT_AFTER);
-
-  self->runner = g_object_ref (runner);
-  self->debugger = g_steal_pointer (&debugger);
-
-  dzl_signal_group_set_target (self->debugger_signals, self->debugger);
+  if (g_set_object (&self->debugger, debugger))
+    ide_signal_group_set_target (self->debugger_signals, self->debugger);
 
   ide_debug_manager_set_active (self, TRUE);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_DEBUGGER]);
 
-  ret = TRUE;
-
-failure:
-  IDE_RETURN (ret);
-}
-
-void
-ide_debug_manager_stop (IdeDebugManager *self)
-{
-  g_return_if_fail (IDE_IS_DEBUG_MANAGER (self));
-
-  dzl_signal_group_set_target (self->debugger_signals, NULL);
-
-  if (self->runner != NULL)
-    {
-      ide_runner_force_quit (self->runner);
-      ide_clear_and_destroy_object (&self->runner);
-    }
-
-  ide_clear_and_destroy_object (&self->debugger);
-  ide_debug_manager_reset_breakpoints (self);
-
-  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_DEBUGGER]);
+  IDE_RETURN (TRUE);
 }
 
 gboolean
@@ -1067,8 +1010,6 @@ ide_debug_manager_get_active (IdeDebugManager *self)
  * Gets the debugger instance, if it is loaded.
  *
  * Returns: (transfer none) (nullable): An #IdeDebugger or %NULL
- *
- * Since: 3.32
  */
 IdeDebugger *
 ide_debug_manager_get_debugger (IdeDebugManager *self)
@@ -1092,8 +1033,6 @@ ide_debug_manager_get_debugger (IdeDebugManager *self)
  * propagate to the debugger when the debugger has been successfully spawned.
  *
  * Returns: (transfer full): An #IdeDebuggerBreakpoints
- *
- * Since: 3.32
  */
 IdeDebuggerBreakpoints *
 ide_debug_manager_get_breakpoints_for_file (IdeDebugManager *self,
@@ -1126,8 +1065,6 @@ ide_debug_manager_get_breakpoints_for_file (IdeDebugManager *self,
  * is done by notifying the debugger to add the breakpoint. If there is
  * not an active debugger, then it is done by caching the breakpoint
  * until the debugger is next started.
- *
- * Since: 3.32
  */
 void
 _ide_debug_manager_add_breakpoint (IdeDebugManager       *self,
@@ -1174,8 +1111,6 @@ _ide_debug_manager_add_breakpoint (IdeDebugManager       *self,
  * This removes an exiting breakpoint. If the debugger has been started, it
  * is done by notifying the debugger to remove the breakpoint. If there is
  * not an active debugger, then it is done by removing the cached breakpoint.
- *
- * Since: 3.32
  */
 void
 _ide_debug_manager_remove_breakpoint (IdeDebugManager       *self,
@@ -1218,8 +1153,6 @@ _ide_debug_manager_remove_breakpoint (IdeDebugManager       *self,
  * Gets the #IdeDebugManager for a context.
  *
  * Returns: (transfer none): an #IdeDebugManager
- *
- * Since: 3.32
  */
 IdeDebugManager *
 ide_debug_manager_from_context (IdeContext *context)
