@@ -28,8 +28,6 @@
  * SECTION:ide-lsp-service
  * @title: IdeLspService
  * @short_description: Service integration for LSPs
- *
- * Since: 42.0
  */
 
 typedef struct
@@ -199,6 +197,7 @@ ide_lsp_service_real_create_launcher (IdeLspService    *self,
 
   IDE_ENTRY;
 
+  g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (IDE_IS_LSP_SERVICE (self));
   g_assert (IDE_IS_PIPELINE (pipeline));
 
@@ -211,11 +210,16 @@ ide_lsp_service_real_create_launcher (IdeLspService    *self,
   /* First try in the build environment */
   if (ide_pipeline_contains_program_in_path (pipeline, priv->program, NULL))
     {
-      if ((launcher = ide_pipeline_create_launcher (pipeline, NULL)))
+      g_autoptr(IdeRunContext) run_context = ide_run_context_new ();
+      IdeRuntime *runtime = ide_pipeline_get_runtime (pipeline);
+
+      ide_runtime_prepare_to_build (runtime, pipeline, run_context);
+      ide_run_context_append_argv (run_context, priv->program);
+      ide_run_context_set_cwd (run_context, srcdir);
+
+      if ((launcher = ide_run_context_end (run_context, NULL)))
         {
           ide_subprocess_launcher_set_flags (launcher, flags);
-          ide_subprocess_launcher_push_argv (launcher, priv->program);
-          ide_subprocess_launcher_set_cwd (launcher, srcdir);
           IDE_RETURN (g_steal_pointer (&launcher));
         }
     }
@@ -228,11 +232,16 @@ ide_lsp_service_real_create_launcher (IdeLspService    *self,
 
       if (ide_runtime_contains_program_in_path (host, priv->program, NULL))
         {
-          if ((launcher = ide_runtime_create_launcher (host, NULL)))
+          g_autoptr(IdeRunContext) run_context = NULL;
+
+          run_context = ide_run_context_new ();
+          ide_runtime_prepare_to_build (host, pipeline, run_context);
+          ide_run_context_append_argv (run_context, priv->program);
+          ide_run_context_set_cwd (run_context, srcdir);
+
+          if ((launcher = ide_run_context_end (run_context, NULL)))
             {
               ide_subprocess_launcher_set_flags (launcher, flags);
-              ide_subprocess_launcher_push_argv (launcher, priv->program);
-              ide_subprocess_launcher_set_cwd (launcher, srcdir);
               IDE_RETURN (g_steal_pointer (&launcher));
             }
         }
@@ -248,11 +257,16 @@ ide_lsp_service_real_create_launcher (IdeLspService    *self,
 
               if (g_file_test (path, G_FILE_TEST_IS_EXECUTABLE))
                 {
-                  if ((launcher = ide_runtime_create_launcher (host, NULL)))
+                  g_autoptr(IdeRunContext) run_context = NULL;
+
+                  run_context = ide_run_context_new ();
+                  ide_runtime_prepare_to_build (host, pipeline, run_context);
+                  ide_run_context_append_argv (run_context, path);
+                  ide_run_context_set_cwd (run_context, srcdir);
+
+                  if ((launcher = ide_run_context_end (run_context, NULL)))
                     {
-                      ide_subprocess_launcher_push_argv (launcher, path);
                       ide_subprocess_launcher_set_flags (launcher, flags);
-                      ide_subprocess_launcher_set_cwd (launcher, srcdir);
                       IDE_RETURN (g_steal_pointer (&launcher));
                     }
                 }
@@ -593,6 +607,9 @@ ide_lsp_service_restart (IdeLspService *self)
   g_return_if_fail (IDE_IS_LSP_SERVICE (self));
   g_return_if_fail (!ide_object_in_destruction (IDE_OBJECT (self)));
 
+  g_debug ("Request to restart LSP service %s",
+           G_OBJECT_TYPE_NAME (self));
+
   ide_lsp_service_stop (self);
 
   if ((context = ide_object_get_context (IDE_OBJECT (self))))
@@ -610,6 +627,13 @@ on_pipeline_loaded_cb (IdeLspService *self,
   g_assert (IDE_IS_LSP_SERVICE (self));
   g_assert (IDE_IS_PIPELINE (pipeline));
 
+  g_signal_handlers_disconnect_by_func (pipeline,
+                                        G_CALLBACK (on_pipeline_loaded_cb),
+                                        self);
+
+  g_debug ("Pipeline has completed loading, restarting LSP service %s",
+           G_OBJECT_TYPE_NAME (self));
+
   ide_lsp_service_restart (self);
 
   IDE_EXIT;
@@ -627,6 +651,9 @@ on_notify_pipeline_cb (IdeLspService   *self,
   g_assert (IDE_IS_LSP_SERVICE (self));
   g_assert (IDE_IS_BUILD_MANAGER (build_manager));
 
+  g_debug ("Pipeline changed, requesting LSP service %s restart",
+           G_OBJECT_TYPE_NAME (self));
+
   ide_lsp_service_stop (self);
 
   if ((pipeline = ide_build_manager_get_pipeline (build_manager)))
