@@ -26,11 +26,25 @@
 #include <webkit2/webkit2.h>
 #include <girepository.h>
 
+#include <libide-core.h>
+
+#include "ide-webkit-page.h"
+
 _IDE_EXTERN void _ide_webkit_register_types (PeasObjectModule *module);
 
 void
 _ide_webkit_register_types (PeasObjectModule *module)
 {
+  WebKitWebContext *context;
+  g_autoptr(GError) error = NULL;
+
   g_type_ensure (WEBKIT_TYPE_WEB_VIEW);
-  g_irepository_require (NULL, "WebKit2", "4.0", 0, NULL);
+  g_type_ensure (IDE_TYPE_WEBKIT_PAGE);
+
+  if (!g_irepository_require (NULL, "WebKit2", "5.0", 0, &error))
+    g_warning ("%s", error->message);
+
+  context = webkit_web_context_get_default ();
+  webkit_web_context_set_sandbox_enabled (context, TRUE);
+  webkit_web_context_set_favicon_database_directory (context, NULL);
 }
