@@ -22,7 +22,6 @@
 
 #include "config.h"
 
-#include <dazzle.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <string.h>
@@ -92,7 +91,7 @@ add_suffix:
 
 static gchar *
 get_next_id (IdeConfigManager *manager,
-             const gchar             *id)
+             const gchar      *id)
 {
   g_autoptr(GPtrArray) tries = NULL;
 
@@ -111,11 +110,11 @@ get_next_id (IdeConfigManager *manager,
 }
 
 static void
-load_string (IdeConfig *config,
-             GKeyFile         *key_file,
-             const gchar      *group,
-             const gchar      *key,
-             const gchar      *property)
+load_string (IdeConfig   *config,
+             GKeyFile    *key_file,
+             const gchar *group,
+             const gchar *key,
+             const gchar *property)
 {
   g_assert (IDE_IS_CONFIG (config));
   g_assert (key_file != NULL);
@@ -133,11 +132,11 @@ load_string (IdeConfig *config,
 }
 
 static void
-load_strv (IdeConfig *config,
-           GKeyFile         *key_file,
-           const gchar      *group,
-           const gchar      *key,
-           const gchar      *property)
+load_strv (IdeConfig   *config,
+           GKeyFile    *key_file,
+           const gchar *group,
+           const gchar *key,
+           const gchar *property)
 {
   g_assert (IDE_IS_CONFIG (config));
   g_assert (key_file != NULL);
@@ -157,19 +156,49 @@ load_strv (IdeConfig *config,
 }
 
 static void
-load_environ (IdeConfig *config,
-              GKeyFile         *key_file,
-              const gchar      *group)
+load_argv (IdeConfig   *config,
+           GKeyFile    *key_file,
+           const gchar *group,
+           const gchar *key,
+           const gchar *property)
+{
+  g_assert (IDE_IS_CONFIG (config));
+  g_assert (key_file != NULL);
+  g_assert (group != NULL);
+  g_assert (key != NULL);
+
+  if (g_key_file_has_key (key_file, group, key, NULL))
+    {
+      g_autofree char *str = NULL;
+      g_auto(GValue) value = G_VALUE_INIT;
+      g_auto(GStrv) argv = NULL;
+      int argc = 0;
+
+      if ((str = g_key_file_get_string (key_file, group, key, NULL)) &&
+          str[0] != 0 &&
+          g_shell_parse_argv (str, &argc, &argv, NULL))
+        {
+          g_value_init (&value, G_TYPE_STRV);
+          g_value_take_boxed (&value, g_steal_pointer (&argv));
+          g_object_set_property (G_OBJECT (config), property, &value);
+        }
+    }
+}
+
+static void
+load_environ (IdeConfig      *config,
+              IdeEnvironment *environment,
+              GKeyFile       *key_file,
+              const gchar    *group)
 {
-  IdeEnvironment *environment;
   g_auto(GStrv) keys = NULL;
   gsize len = 0;
 
   g_assert (IDE_IS_CONFIG (config));
+  g_assert (IDE_IS_ENVIRONMENT (environment));
   g_assert (key_file != NULL);
   g_assert (group != NULL);
 
-  environment = ide_config_get_environment (config);
   keys = g_key_file_get_keys (key_file, group, &len, NULL);
 
   for (gsize i = 0; i < len; i++)
@@ -184,10 +213,13 @@ load_environ (IdeConfig *config,
 
 static IdeConfig *
 ide_buildconfig_config_provider_create (IdeBuildconfigConfigProvider *self,
-                                               const gchar                         *config_id)
+                                        const gchar                  *config_id)
 {
   g_autoptr(IdeConfig) config = NULL;
   g_autofree gchar *env_group = NULL;
+  g_autofree gchar *rt_env_group = NULL;
+  IdeEnvironment   *environment;
+  IdeEnvironment   *rt_environment;
 
   g_assert (IDE_IS_BUILDCONFIG_CONFIG_PROVIDER (self));
   g_assert (self->key_file != NULL);
@@ -207,6 +239,7 @@ ide_buildconfig_config_provider_create (IdeBuildconfigConfigProvider *self,
   load_string (config, self->key_file, config_id, "app-id", "app-id");
   load_strv (config, self->key_file, config_id, "prebuild", "prebuild");
   load_strv (config, self->key_file, config_id, "postbuild", "postbuild");
+  load_argv (config, self->key_file, config_id, "run-command", "run-command");
 
   if (g_key_file_has_key (self->key_file, config_id, "builddir", NULL))
     {
@@ -218,16 +251,26 @@ ide_buildconfig_config_provider_create (IdeBuildconfigConfigProvider *self,
 
   env_group = g_strdup_printf ("%s.environment", config_id);
   if (g_key_file_has_group (self->key_file, env_group))
-    load_environ (config, self->key_file, env_group);
+    {
+      environment = ide_config_get_environment (config);
+      load_environ (config, environment, self->key_file, env_group);
+    }
+
+  rt_env_group = g_strdup_printf ("%s.runtime_environment", config_id);
+  if (g_key_file_has_group (self->key_file, rt_env_group))
+    {
+      rt_environment = ide_config_get_runtime_environment (config);
+      load_environ (config, rt_environment, self->key_file, rt_env_group);
+    }
 
   return g_steal_pointer (&config);
 }
 
 static void
-ide_buildconfig_config_provider_load_async (IdeConfigProvider *provider,
-                                                   GCancellable             *cancellable,
-                                                   GAsyncReadyCallback       callback,
-                                                   gpointer                  user_data)
+ide_buildconfig_config_provider_load_async (IdeConfigProvider   *provider,
+                                            GCancellable        *cancellable,
+                                            GAsyncReadyCallback  callback,
+                                            gpointer             user_data)
 {
   IdeBuildconfigConfigProvider *self = (IdeBuildconfigConfigProvider *)provider;
   g_autoptr(IdeConfig) fallback = NULL;
@@ -301,8 +344,8 @@ complete:
 
 static gboolean
 ide_buildconfig_config_provider_load_finish (IdeConfigProvider  *provider,
-                                                    GAsyncResult              *result,
-                                                    GError                   **error)
+                                             GAsyncResult       *result,
+                                             GError            **error)
 {
   g_assert (IDE_IS_BUILDCONFIG_CONFIG_PROVIDER (provider));
   g_assert (IDE_IS_TASK (result));
@@ -311,10 +354,53 @@ ide_buildconfig_config_provider_load_finish (IdeConfigProvider  *provider,
   return ide_task_propagate_boolean (IDE_TASK (result), error);
 }
 
+static void
+ide_buildconfig_config_provider_append_env (IdeEnvironment *env,
+                                            gchar          *env_group,
+                                            GKeyFile       *key_file)
+{
+  guint n_items;
+
+  /*
+   * Remove all environment keys that are no longer specified in the
+   * environment. This allows us to just do a single pass of additions
+   * from the environment below.
+   */
+  if (g_key_file_has_group (key_file, env_group))
+    {
+      g_auto(GStrv) keys = NULL;
+
+      if (NULL != (keys = g_key_file_get_keys (key_file, env_group, NULL, NULL)))
+        {
+          for (guint j = 0; keys [j]; j++)
+            {
+              if (!ide_environment_getenv (env, keys [j]))
+                g_key_file_remove_key (key_file, env_group, keys [j], NULL);
+            }
+        }
+    }
+
+  n_items = g_list_model_get_n_items (G_LIST_MODEL (env));
+
+  for (guint j = 0; j < n_items; j++)
+    {
+      g_autoptr(IdeEnvironmentVariable) var = NULL;
+      const gchar *key;
+      const gchar *value;
+
+      var = g_list_model_get_item (G_LIST_MODEL (env), j);
+      key = ide_environment_variable_get_key (var);
+      value = ide_environment_variable_get_value (var);
+
+      if (!ide_str_empty0 (key))
+        g_key_file_set_string (key_file, env_group, key, value ?: "");
+    }
+}
+
 static void
 ide_buildconfig_config_provider_save_cb (GObject      *object,
-                                                GAsyncResult *result,
-                                                gpointer      user_data)
+                                         GAsyncResult *result,
+                                         gpointer      user_data)
 {
   GFile *file = (GFile *)object;
   g_autoptr(IdeTask) task = user_data;
@@ -392,15 +478,17 @@ ide_buildconfig_config_provider_save_async (IdeConfigProvider   *provider,
     {
       IdeConfig *config = g_ptr_array_index (self->configs, i);
       g_autofree gchar *env_group = NULL;
+      g_autofree gchar *rt_env_group = NULL;
       const gchar *config_id;
       IdeEnvironment *env;
-      guint n_items;
+      IdeEnvironment *rt_env;
 
       if (!ide_config_get_dirty (config))
         continue;
 
       config_id = ide_config_get_id (config);
       env_group = g_strdup_printf ("%s.environment", config_id);
+      rt_env_group = g_strdup_printf ("%s.runtime_environment", config_id);
 
       /*
        * Track our known group names, so we can remove missing names after
@@ -408,6 +496,7 @@ ide_buildconfig_config_provider_save_async (IdeConfigProvider   *provider,
        */
       g_hash_table_insert (group_names, g_strdup (config_id), NULL);
       g_hash_table_insert (group_names, g_strdup (env_group), NULL);
+      g_hash_table_insert (group_names, g_strdup (rt_env_group), NULL);
 
 #define PERSIST_STRING_KEY(key, getter) \
       g_key_file_set_string (self->key_file, config_id, key, \
@@ -417,6 +506,16 @@ ide_buildconfig_config_provider_save_async (IdeConfigProvider   *provider,
       gsize vlen = val ? g_strv_length ((gchar **)val) : 0; \
       g_key_file_set_string_list (self->key_file, config_id, key, val, vlen); \
 } G_STMT_END
+#define PERSIST_ARGV_KEY(key, getter) G_STMT_START { \
+      const gchar * const *val = ide_buildconfig_config_##getter (IDE_BUILDCONFIG_CONFIG (config)); \
+      if (val) \
+        { \
+          g_autofree char *str = g_strjoinv (" ", (char **)val); \
+          g_key_file_set_string (self->key_file, config_id, key, str); \
+        } \
+      else \
+        g_key_file_set_string (self->key_file, config_id, key, ""); \
+} G_STMT_END
 
       PERSIST_STRING_KEY ("name", get_display_name);
       PERSIST_STRING_KEY ("runtime", get_runtime_id);
@@ -427,6 +526,7 @@ ide_buildconfig_config_provider_save_async (IdeConfigProvider   *provider,
       PERSIST_STRING_KEY ("app-id", get_app_id);
       PERSIST_STRV_KEY ("postbuild", get_postbuild);
       PERSIST_STRV_KEY ("prebuild", get_prebuild);
+      PERSIST_ARGV_KEY ("run-command", get_run_command);
 
 #undef PERSIST_STRING_KEY
 #undef PERSIST_STRV_KEY
@@ -444,41 +544,10 @@ ide_buildconfig_config_provider_save_async (IdeConfigProvider   *provider,
         g_key_file_remove_key (self->key_file, config_id, "default", NULL);
 
       env = ide_config_get_environment (config);
+      ide_buildconfig_config_provider_append_env(env, env_group, self->key_file);
 
-      /*
-       * Remove all environment keys that are no longer specified in the
-       * environment. This allows us to just do a single pass of additions
-       * from the environment below.
-       */
-      if (g_key_file_has_group (self->key_file, env_group))
-        {
-          g_auto(GStrv) keys = NULL;
-
-          if (NULL != (keys = g_key_file_get_keys (self->key_file, env_group, NULL, NULL)))
-            {
-              for (guint j = 0; keys [j]; j++)
-                {
-                  if (!ide_environment_getenv (env, keys [j]))
-                    g_key_file_remove_key (self->key_file, env_group, keys [j], NULL);
-                }
-            }
-        }
-
-      n_items = g_list_model_get_n_items (G_LIST_MODEL (env));
-
-      for (guint j = 0; j < n_items; j++)
-        {
-          g_autoptr(IdeEnvironmentVariable) var = NULL;
-          const gchar *key;
-          const gchar *value;
-
-          var = g_list_model_get_item (G_LIST_MODEL (env), j);
-          key = ide_environment_variable_get_key (var);
-          value = ide_environment_variable_get_value (var);
-
-          if (!dzl_str_empty0 (key))
-            g_key_file_set_string (self->key_file, env_group, key, value ?: "");
-        }
+      rt_env = ide_config_get_runtime_environment (config);
+      ide_buildconfig_config_provider_append_env(rt_env, rt_env_group, self->key_file);
 
       ide_config_set_dirty (config, FALSE);
     }
