@@ -22,17 +22,15 @@
 
 #include "config.h"
 
-#include <dazzle.h>
-
 #include "gbp-buildui-stage-row.h"
 
 struct _GbpBuilduiStageRow
 {
-  GtkListBoxRow    parent_instance;
+  GtkListBoxRow       parent_instance;
 
   IdePipelineStage   *stage;
 
-  DzlBoldingLabel *label;
+  GtkLabel           *label;
 };
 
 enum {
@@ -41,27 +39,41 @@ enum {
   N_PROPS
 };
 
-G_DEFINE_TYPE (GbpBuilduiStageRow, gbp_buildui_stage_row, GTK_TYPE_LIST_BOX_ROW)
+G_DEFINE_FINAL_TYPE (GbpBuilduiStageRow, gbp_buildui_stage_row, GTK_TYPE_LIST_BOX_ROW)
 
 static GParamSpec *properties [N_PROPS];
 
+static void
+gbp_buildui_stage_row_notify_active (GbpBuilduiStageRow *row,
+                                     GParamSpec         *pspec,
+                                     IdePipelineStage   *stage)
+{
+  g_assert (GBP_IS_BUILDUI_STAGE_ROW (row));
+  g_assert (IDE_IS_PIPELINE_STAGE (stage));
+
+  if (ide_pipeline_stage_get_active (stage))
+    gtk_widget_add_css_class (GTK_WIDGET (row->label), "heading");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (row->label), "heading");
+}
+
 static void
 gbp_buildui_stage_row_notify_completed (GbpBuilduiStageRow *row,
-                                      GParamSpec       *pspec,
-                                      IdePipelineStage    *stage)
+                                        GParamSpec         *pspec,
+                                        IdePipelineStage   *stage)
 {
   g_assert (GBP_IS_BUILDUI_STAGE_ROW (row));
   g_assert (IDE_IS_PIPELINE_STAGE (stage));
 
   if (ide_pipeline_stage_get_completed (stage))
-    dzl_gtk_widget_add_style_class (GTK_WIDGET (row->label), "dim-label");
+    gtk_widget_add_css_class (GTK_WIDGET (row->label), "dim-label");
   else
-    dzl_gtk_widget_remove_style_class (GTK_WIDGET (row->label), "dim-label");
+    gtk_widget_remove_css_class (GTK_WIDGET (row->label), "dim-label");
 }
 
 static void
 gbp_buildui_stage_row_set_stage (GbpBuilduiStageRow *self,
-                               IdePipelineStage    *stage)
+                                 IdePipelineStage   *stage)
 {
   const gchar *name;
 
@@ -75,7 +87,7 @@ gbp_buildui_stage_row_set_stage (GbpBuilduiStageRow *self,
   if (name == NULL)
     name = G_OBJECT_TYPE_NAME (stage);
 
-  gtk_label_set_label (GTK_LABEL (self->label), name);
+  gtk_label_set_label (self->label, name);
 
   g_signal_connect_object (stage,
                            "notify::completed",
@@ -83,27 +95,35 @@ gbp_buildui_stage_row_set_stage (GbpBuilduiStageRow *self,
                            self,
                            G_CONNECT_SWAPPED);
 
+  g_signal_connect_object (stage,
+                           "notify::active",
+                           G_CALLBACK (gbp_buildui_stage_row_notify_active),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   g_object_bind_property (stage, "disabled", self, "sensitive", G_BINDING_DEFAULT);
+#if 0
   g_object_bind_property (stage, "active", self->label, "bold", G_BINDING_DEFAULT);
+#endif
 
   gbp_buildui_stage_row_notify_completed (self, NULL, stage);
 }
 
 static void
-gbp_buildui_stage_row_destroy (GtkWidget *widget)
+gbp_buildui_stage_row_dispose (GObject *object)
 {
-  GbpBuilduiStageRow *self = (GbpBuilduiStageRow *)widget;
+  GbpBuilduiStageRow *self = (GbpBuilduiStageRow *)object;
 
   g_clear_object (&self->stage);
 
-  GTK_WIDGET_CLASS (gbp_buildui_stage_row_parent_class)->destroy (widget);
+  G_OBJECT_CLASS (gbp_buildui_stage_row_parent_class)->dispose (object);
 }
 
 static void
 gbp_buildui_stage_row_get_property (GObject    *object,
-                                  guint       prop_id,
-                                  GValue     *value,
-                                  GParamSpec *pspec)
+                                    guint       prop_id,
+                                    GValue     *value,
+                                    GParamSpec *pspec)
 {
   GbpBuilduiStageRow *self = GBP_BUILDUI_STAGE_ROW (object);
 
@@ -120,9 +140,9 @@ gbp_buildui_stage_row_get_property (GObject    *object,
 
 static void
 gbp_buildui_stage_row_set_property (GObject      *object,
-                                  guint         prop_id,
-                                  const GValue *value,
-                                  GParamSpec   *pspec)
+                                    guint         prop_id,
+                                    const GValue *value,
+                                    GParamSpec   *pspec)
 {
   GbpBuilduiStageRow *self = GBP_BUILDUI_STAGE_ROW (object);
 
@@ -143,11 +163,10 @@ gbp_buildui_stage_row_class_init (GbpBuilduiStageRowClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->dispose = gbp_buildui_stage_row_dispose;
   object_class->get_property = gbp_buildui_stage_row_get_property;
   object_class->set_property = gbp_buildui_stage_row_set_property;
 
-  widget_class->destroy = gbp_buildui_stage_row_destroy;
-
   properties [PROP_STAGE] =
     g_param_spec_object ("stage",
                          "Stage",
@@ -175,7 +194,6 @@ gbp_buildui_stage_row_new (IdePipelineStage *stage)
 
   return g_object_new (GBP_TYPE_BUILDUI_STAGE_ROW,
                        "stage", stage,
-                       "visible", TRUE,
                        NULL);
 }
 
@@ -186,8 +204,6 @@ gbp_buildui_stage_row_new (IdePipelineStage *stage)
  * Gets the stage for the row.
  *
  * Returns: (transfer none): an #IdePipelineStage
- *
- * Since: 3.32
  */
 IdePipelineStage *
 gbp_buildui_stage_row_get_stage (GbpBuilduiStageRow *self)
