@@ -48,18 +48,16 @@ ide_code_index_search_result_activate (IdeSearchResult *result,
                                        GtkWidget       *last_focus)
 {
   IdeCodeIndexSearchResult *self = (IdeCodeIndexSearchResult *)result;
+  g_autoptr(PanelPosition) position = NULL;
   IdeWorkspace *workspace;
-  IdeSurface *editor;
 
   g_assert (IDE_IS_CODE_INDEX_SEARCH_RESULT (self));
   g_assert (GTK_IS_WIDGET (last_focus));
 
-  if (!last_focus)
-    return;
+  workspace = ide_widget_get_workspace (last_focus);
+  position = panel_position_new ();
 
-  if ((workspace = ide_widget_get_workspace (last_focus)) &&
-      (editor = ide_workspace_get_surface_by_name (workspace, "editor")))
-    ide_editor_surface_focus_location (IDE_EDITOR_SURFACE (editor), self->location);
+  ide_editor_focus_location (workspace, position, self->location);
 }
 
 static void
@@ -138,18 +136,18 @@ ide_code_index_search_result_init (IdeCodeIndexSearchResult *self)
 }
 
 IdeCodeIndexSearchResult *
-ide_code_index_search_result_new (const gchar *title,
-                                  const gchar *subtitle,
-                                  const gchar *icon_name,
+ide_code_index_search_result_new (const char  *title,
+                                  const char  *subtitle,
+                                  GIcon       *gicon,
                                   IdeLocation *location,
-                                  gfloat       score)
+                                  float        score)
 {
   g_autofree gchar *etitle = g_markup_escape_text (title, -1);
 
   return g_object_new (IDE_TYPE_CODE_INDEX_SEARCH_RESULT,
                        "title", etitle,
                        "subtitle", subtitle,
-                       "icon-name", icon_name,
+                       "gicon", gicon,
                        "location", location,
                        "score", score,
                        NULL);
