@@ -41,22 +41,26 @@ struct _GbpFileSearchProvider
 static void search_provider_iface_init (IdeSearchProviderInterface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (GbpFileSearchProvider,
-                         gbp_file_search_provider,
-                         IDE_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (IDE_TYPE_SEARCH_PROVIDER, search_provider_iface_init))
+                               gbp_file_search_provider,
+                               IDE_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (IDE_TYPE_SEARCH_PROVIDER, search_provider_iface_init))
 
 static void
 gbp_file_search_provider_search_async (IdeSearchProvider   *provider,
-                                      const gchar         *search_terms,
-                                      guint                max_results,
-                                      GCancellable        *cancellable,
-                                      GAsyncReadyCallback  callback,
-                                      gpointer             user_data)
+                                       const char          *search_terms,
+                                       guint                max_results,
+                                       GCancellable        *cancellable,
+                                       GAsyncReadyCallback  callback,
+                                       gpointer             user_data)
 {
   GbpFileSearchProvider *self = (GbpFileSearchProvider *)provider;
-  g_autoptr(IdeTask) task = NULL;
+  g_autoptr(GListStore) store = NULL;
   g_autoptr(GPtrArray) results = NULL;
+  g_autoptr(IdeTask) task = NULL;
 
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (GBP_IS_FILE_SEARCH_PROVIDER (self));
   g_assert (search_terms != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
@@ -65,33 +69,48 @@ gbp_file_search_provider_search_async (IdeSearchProvider   *provider,
   ide_task_set_source_tag (task, gbp_file_search_provider_search_async);
   ide_task_set_priority (task, G_PRIORITY_LOW);
 
-  if (self->index != NULL)
-    results = gbp_file_search_index_populate (self->index, search_terms, max_results);
-  else
-    results = g_ptr_array_new_with_free_func (g_object_unref);
+  if (self->index == NULL ||
+      !(results = gbp_file_search_index_populate (self->index, search_terms, max_results)))
+    {
+      ide_task_return_unsupported_error (task);
+      IDE_EXIT;
+    }
 
-  ide_task_return_pointer (task, g_steal_pointer (&results), g_ptr_array_unref);
+  if (results->len >= max_results)
+    g_object_set_data (G_OBJECT (task), "TRUNCATED", GINT_TO_POINTER (TRUE));
+
+  store = g_list_store_new (IDE_TYPE_SEARCH_RESULT);
+  g_list_store_splice (store, 0, 0, results->pdata, results->len);
+  ide_task_return_pointer (task, g_steal_pointer (&store), g_object_unref);
+
+  IDE_EXIT;
 }
 
-static GPtrArray *
+static GListModel *
 gbp_file_search_provider_search_finish (IdeSearchProvider  *provider,
-                                       GAsyncResult       *result,
-                                       GError            **error)
+                                        GAsyncResult       *result,
+                                        gboolean           *truncated,
+                                        GError            **error)
 {
-  GPtrArray *ret;
+  GListModel *ret;
 
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (GBP_IS_FILE_SEARCH_PROVIDER (provider));
   g_assert (IDE_IS_TASK (result));
 
   ret = ide_task_propagate_pointer (IDE_TASK (result), error);
 
-  return IDE_PTR_ARRAY_STEAL_FULL (&ret);
+  *truncated = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (result), "TRUNCATED"));
+
+  IDE_RETURN (ret);
 }
 
 static void
 on_buffer_loaded (GbpFileSearchProvider *self,
-                  IdeBuffer            *buffer,
-                  IdeBufferManager     *bufmgr)
+                  IdeBuffer             *buffer,
+                  IdeBufferManager      *bufmgr)
 {
   g_autofree gchar *relative_path = NULL;
   g_autoptr(IdeContext) context = NULL;
@@ -120,9 +139,9 @@ on_buffer_loaded (GbpFileSearchProvider *self,
 
 static void
 on_file_renamed (GbpFileSearchProvider *self,
-                 GFile                *src_file,
-                 GFile                *dst_file,
-                 IdeProject           *project)
+                 GFile                 *src_file,
+                 GFile                 *dst_file,
+                 IdeProject            *project)
 {
   g_autofree gchar *old_path = NULL;
   g_autofree gchar *new_path = NULL;
@@ -147,8 +166,8 @@ on_file_renamed (GbpFileSearchProvider *self,
 
 static void
 on_file_trashed (GbpFileSearchProvider *self,
-                 GFile                *file,
-                 IdeProject           *project)
+                 GFile                 *file,
+                 IdeProject            *project)
 {
   g_autofree gchar *path = NULL;
   g_autoptr(GFile) workdir = NULL;
@@ -170,8 +189,8 @@ on_file_trashed (GbpFileSearchProvider *self,
 
 static void
 gbp_file_search_provider_build_cb (GObject      *object,
-                                  GAsyncResult *result,
-                                  gpointer      user_data)
+                                   GAsyncResult *result,
+                                   gpointer      user_data)
 {
   GbpFileSearchIndex *index = (GbpFileSearchIndex *)object;
   g_autoptr(GbpFileSearchProvider) self = user_data;
